@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include "tasklist.h"
 #include "window.h"
+#include "workspace.h"
 #include "application.h"
 #include "xutils.h"
 
@@ -108,7 +109,9 @@ struct _WnckTasklistPrivate
   gint max_button_width;
   gint max_button_height;
 
-  gboolean grouping_enabled;
+  gboolean switch_workspace_on_unminimize;
+  
+  WnckTasklistGroupingType grouping;
   gint grouping_limit;
 
   guint activate_timeout_id;
@@ -332,7 +335,7 @@ wnck_tasklist_init (WnckTasklist *tasklist)
   tasklist->priv->win_hash = g_hash_table_new (NULL, NULL);
   tasklist->priv->app_hash = g_hash_table_new (NULL, NULL);
   
-  tasklist->priv->grouping_enabled = TRUE;
+  tasklist->priv->grouping = WNCK_TASKLIST_AUTO_GROUP;
   tasklist->priv->grouping_limit = DEFAULT_GROUPING_LIMIT;
 }
 
@@ -383,20 +386,27 @@ wnck_tasklist_finalize (GObject *object)
 }
 
 void
-wnck_tasklist_set_allow_grouping (WnckTasklist *tasklist,
-				  gboolean allow_grouping)
+wnck_tasklist_set_grouping (WnckTasklist            *tasklist,
+			    WnckTasklistGroupingType grouping)
 {
   g_return_if_fail (WNCK_IS_TASKLIST (tasklist));
 
-  allow_grouping = (allow_grouping != 0);
-
-  if (tasklist->priv->grouping_enabled == allow_grouping)
+  if (tasklist->priv->grouping == grouping)
     return;
   
-  tasklist->priv->grouping_enabled = allow_grouping;
+  tasklist->priv->grouping = grouping;
   gtk_widget_queue_resize (GTK_WIDGET (tasklist));
 }
 
+void
+wnck_tasklist_set_switch_workspace_on_unminimize (WnckTasklist  *tasklist,
+						  gboolean       switch_workspace_on_unminimize)
+{
+  g_return_if_fail (WNCK_IS_TASKLIST (tasklist));
+
+  tasklist->priv->switch_workspace_on_unminimize = switch_workspace_on_unminimize;
+}
+
 void
 wnck_tasklist_set_include_all_workspaces (WnckTasklist *tasklist,
 					  gboolean      include_all_workspaces)
@@ -637,9 +647,10 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
 				       tasklist->priv->max_button_height,
 				       n_windows,
 				       &n_cols, &n_rows);
-  while (tasklist->priv->grouping_enabled &&
-	 (ungrouped_apps != NULL) &&
-	 (button_width < tasklist->priv->grouping_limit))
+  while (ungrouped_apps != NULL &&
+	 ((tasklist->priv->grouping == WNCK_TASKLIST_ALWAYS_GROUP) ||
+	  ((tasklist->priv->grouping == WNCK_TASKLIST_AUTO_GROUP) &&
+	   (button_width < tasklist->priv->grouping_limit))))
     {
       if (!score_set)
 	{
@@ -1110,7 +1121,15 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
 
   if (state & WNCK_WINDOW_STATE_MINIMIZED)
     {
-      wnck_window_unminimize (task->window);
+      WnckWorkspace *active_ws;
+      WnckWorkspace *window_ws;
+
+      active_ws = wnck_screen_get_active_workspace (tasklist->priv->screen);
+      window_ws = wnck_window_get_workspace (task->window);
+      if (active_ws != window_ws &&
+	  !tasklist->priv->switch_workspace_on_unminimize)
+	wnck_workspace_activate (window_ws);
+	  
       wnck_window_activate (task->window);
     }
   else
